@@ -4,7 +4,7 @@ import os
 import time
 from datetime import datetime
 from pathlib import Path
-from typing import BinaryIO, List, Optional, Union
+from typing import BinaryIO, Set, List, Optional, Union
 from urllib.parse import urlparse
 
 import boto3
@@ -67,18 +67,25 @@ def get_dataset(dataset_id: str, token: Optional[str] = None) -> Optional[Datase
     if response.status_code == 200:
         # print("get datasets ", response.json())
         files = convert_to_dataset_file(response.json().get("Files"))
+        # print("Get Files ", files)
         files_with_subpath = convert_to_dataset_file(
             response.json().get("FilesStructure")
         )
+        # print("Get Files with subpath", files_with_subpath)
+        # filter the files overlapped with files_with_subpath
+        files_key_set = set([file.key for file in files])
+        for file in files:
+            if file.key not in files_key_set:
+                files_with_subpath.append(file)
         # print("files_with_subpath", files_with_subpath)
-        files.extend(files_with_subpath)
+        # files.extend(files_with_subpath)
         # print("return dataset files",files)
         return Dataset(
             _id=response.json().get("_id"),
             title=response.json().get("Title"),
             short_description=response.json().get("ShortDescription"),
             folder_name=response.json().get("FolderName"),
-            files=files,
+            files=files_with_subpath,
             created_at=parse_datetime(response.json().get("CreateDate")),
             updated_at=parse_datetime(response.json().get("UpdateDate")),
         )
@@ -104,12 +111,6 @@ def _update_dataset(id: str, files: List[DatasetFile], token: Optional[str] = No
             )
     url = f"{HEYWHALE_SITE}/api/datasets/{id}/files"
     headers = {"x-kesci-token": token, "x-kesci-resource": id}
-    data = {
-        "Files": [file.key for file in files],
-        "FilesStructure": [
-            {"Token": file.key, "SubPath": file.sub_path} for file in files
-        ],
-    }
     response = requests.put(
         url,
         json={
@@ -176,7 +177,7 @@ def _upload_file(
     path_or_fileobj: Union[str, Path, bytes, BinaryIO],
     path_in_dataset: str | Path,
     upload_info: UploadInfo,
-)->str:
+) -> str:
     """
     Uploads a file to a dataset.
 
@@ -202,7 +203,11 @@ def _upload_file(
         upload_info.prefix_to_save, generate_timestamped_string(1)
     )
     if path_in_dataset != "":
-        object_key = os.path.join(object_key, path_in_dataset)
+        # get base filename from path_in_dataset
+        # 前端是在对象存储中key没有带目录的
+        # 目录存在api的SubPath字段里面
+        # 这里仿照前端的格式存以防有什么不统一的地方导致错误
+        object_key = os.path.join(object_key, os.path.basename(path_in_dataset))
     # print(f"upload object key {object_key}")
     try:
         if isinstance(path_or_fileobj, (str, Path)):
@@ -214,6 +219,7 @@ def _upload_file(
         print(f"Error putting object '{object_key}' from bucket '{bucket_name}': {e}")
     return object_key
 
+
 def upload_file(
     path_or_fileobj: Union[str, Path, bytes, BinaryIO],
     path_in_dataset: str | Path,
@@ -247,28 +253,41 @@ def upload_file(
     dataset = get_dataset(id, token)
     if dataset is not None:
         for file in dataset.files:
-            if _get_file_in_dataset(upload_info.prefix_to_save, file.key) != path_in_dataset:
+            if file.path_in_dataset() != path_in_dataset:
                 new_dataset_files.append(file)
             elif not overwrite:
                 # print("file exists, skip uploading")
                 return
         pp = str(Path(path_in_dataset).parent)
-        # 强加一个 /，按照示例格式来，怕没加没处理好。
-        if not pp.endswith("/"):
-            pp = pp + "/"
-        # 开头的也去掉，避免绝对路径的格式。
-        if pp.startswith("/"):
-            pp = pp[1:]
-        # 本地目录直接忽略
-        if pp.startswith("./"):
-            pp = pp[2:]
-        new_dataset_files.append(DatasetFile("", object_key, 0, pp))
-        _update_dataset(id, new_dataset_files, token)
+        new_dataset_files.append(DatasetFile("", object_key, 0, _norm_path(pp)))
+    _update_dataset(id, new_dataset_files, token)
+
+
+def _norm_path(pp: str) -> str:
+    """让目录格式和前端传的一样
+
+    Args:
+        path (str): /a/ -> a/ ./ -> "" /a -> a/
+
+    Returns:
+        str: 带尾部"/"的相对路径
+    """
+    print(f"input pp {pp}")
+    # 强加一个 /，按照示例格式来，怕没加没处理好。
+    if not pp.endswith("/"):
+        pp = pp + "/"
+    # 开头的也去掉，避免绝对路径的格式。
+    if pp.startswith("/"):
+        pp = pp[1:]
+    # 本地目录直接忽略
+    if pp.startswith("./"):
+        pp = pp[2:]
+    return pp
 
 
 def upload_folder(
     folder_path: str | Path,
-    path_in_dataset: str,
+    folder_in_dataset: str,
     id: str | Dataset,
     overwrite: bool = False,
     token: Optional[str] = None,
@@ -284,52 +303,44 @@ def upload_folder(
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
-    new_dataset_files: List[DatasetFile] = []
+    new_dataset_files: Set[DatasetFile] = set()
     dataset = get_dataset(id, token)
     if dataset is None:
         raise ValueError(f"Dataset '{id}' not found.")
     # Walk through the directory
+    # print(f"folder path {folder_path}")
     for root, _dirs, files in os.walk(folder_path):
+        # print("files")
         for file in files:
+            # print(f"file {file}")
             file_path = os.path.join(root, file)
             rel_path = os.path.relpath(os.path.join(root, file), folder_path)
+            path_in_dataset = os.path.join(folder_in_dataset, rel_path)
             # print(f"root {root} rel_path {rel_path} upload file {file_path} path in {os.path.join(path_in_dataset,rel_path)} id {id}")
-            object_key = _upload_file(
-                file_path, os.path.join(path_in_dataset, rel_path), upload_info)
+            object_key = _upload_file(file_path, path_in_dataset, upload_info)
+            skip_upload = False
             for dfile in dataset.files:
-                if _get_file_in_dataset(upload_info.prefix_to_save, dfile.key) != path_in_dataset:
-                    new_dataset_files.append(dfile)
+                # print(
+                #     f"dfile {dfile.path_in_dataset() } file {path_in_dataset}"
+                # )
+                if dfile.path_in_dataset() != path_in_dataset:
+                    # print(f"add dfile {dfile}")
+                    new_dataset_files.add(dfile)
                 elif not overwrite:
                     # print("file exists, skip uploading")
+                    skip_upload = True
                     break
+            if skip_upload:
+                break
+            else:
                 pp = str(Path(path_in_dataset).parent)
-                # 强加一个 /，按照示例格式来，怕没加没处理好。
-                if not pp.endswith("/"):
-                    pp = pp + "/"
-                # 开头的也去掉，避免绝对路径的格式。
-                if pp.startswith("/"):
-                    pp = pp[1:]
-                # 本地目录直接忽略
-                if pp.startswith("./"):
-                    pp = pp[2:]
-                new_dataset_files.append(DatasetFile("", object_key, 0, pp))
-    _update_dataset(id, new_dataset_files, token)
-
-def _get_file_in_dataset(prefix: str, object_key: str) -> str:
-    """
-    Extracts the file path from the object key in a dataset.
-
-    :param prefix: The prefix of the object key.
-    :param object_key: The object key.
-    :return: The file path.
-    """
-    if object_key.startswith(prefix):
-        prefix_end_index = len(prefix)
-        next_slash_index = object_key.find("/", prefix_end_index)
-        file_path = object_key[next_slash_index + 1 :]
-        return file_path
-    else:
-        return ""
+                # print(f"Append new file {object_key}, {_norm_path(pp)}")
+                new_dataset_files.add(DatasetFile("", object_key, 0, _norm_path(pp)))
+    if len(new_dataset_files) == 0:
+        # skip update if no files in directory
+        return
+    # print(new_dataset_files)
+    _update_dataset(id, list(new_dataset_files), token)
 
 
 def _init_cache(cache_dir: Optional[str | Path] = None) -> Path:
@@ -386,7 +397,10 @@ def _download_single_file(
     )
     file_path.parent.mkdir(parents=True, exist_ok=True)
     if file_path.exists():
-        pass
+        raise ValueError(
+            f"File '{filename}' is being downloaded, please wait. If the downloading process is interrupted, please delete the lock file '{file_path}' manually."
+        )
+
     m = hashlib.md5()
     with open(file_path, "wb") as file:
         for chunk in response.iter_content(chunk_size=chunk_size):
@@ -513,6 +527,44 @@ def download_file(
     return slk_path
 
 
+def delete_file(
+    id: str,
+    filepath_in_dataset: str,
+    token: Optional[str] = None,
+):
+    """Delete a file from the dataset."""
+    if token is None:
+        token = os.getenv("MW_TOKEN")
+        if not token:
+            raise ValueError(
+                "No token provided and 'MW_TOKEN' environment variable is not set."
+            )
+    dataset = get_dataset(id, token)
+    if dataset is None:
+        raise ValueError(f"Dataset '{id}' not found.")
+    upload_info = _get_update_token(token)
+    if upload_info is None:
+        raise ValueError("Failed to get upload token.")
+    # print("HHHHH")
+    new_dataset_files: List[DatasetFile] = []
+    # print("dateset files", dataset.files)
+    for file in dataset.files:
+        # a = file.path_in_dataset()
+        # b = filepath_in_dataset
+        # print(f"A {a} B {b}")
+        if file.path_in_dataset() != filepath_in_dataset:
+            new_dataset_files.append(file)
+        else:
+            print(f"delete file {file.path_in_dataset()}")
+            continue
+    _update_dataset(id, new_dataset_files, token)
+    return
+
+
+def delete_folder():
+    pass
+
+
 def create_dataset(
     title: str,
     short_description: Optional[str] = None,
@@ -598,6 +650,36 @@ def create_dataset(
         return None
 
 
+def delete_dataset(
+    id: str | Dataset,
+    token: Optional[str] = None,
+):
+    if token is None:
+        token = os.getenv("MW_TOKEN")
+        if not token:
+            raise ValueError(
+                "No token provided and 'MW_TOKEN' environment variable is not set."
+            )
+    if isinstance(id, Dataset):
+        id = id._id
+    url = f"{HEYWHALE_SITE}/api/datasets/{id}"
+    headers = {
+        "x-kesci-token": token,
+        "x-kesci-resource": id,
+    }
+    response = requests.delete(
+        url,
+        headers=headers,
+        timeout=10,
+    )
+    if response.status_code == 200:
+        return
+    else:
+        print(response.text)
+        response.raise_for_status()
+        return
+
+
 def list_datasets(
     title: str = "",
     limit: int = 10,
@@ -632,14 +714,17 @@ def list_datasets(
         for d in document.get("data"):
             files = convert_to_dataset_file(d.get("Files"))
             files_with_subpath = convert_to_dataset_file(d.get("FilesStructure"))
-            files.extend(files_with_subpath)
+            files_key_set = set([file.key for file in files])
+            for file in files:
+                if file.key not in files_key_set:
+                    files_with_subpath.append(file)
             datasets.append(
                 Dataset(
                     _id=d.get("_id"),
                     title=d.get("Title"),
                     short_description=d.get("ShortDescription"),
                     folder_name=d.get("FolderName"),
-                    files=files,
+                    files=files_with_subpath,
                     created_at=parse_datetime(d.get("CreateDate")),
                     updated_at=parse_datetime(d.get("UpdateDate")),
                 )
