@@ -1,5 +1,5 @@
 import unittest
-from mw_python_sdk import create_dataset,upload_file
+from mw_python_sdk import create_dataset, delete_dataset, upload_file
 
 
 class TestCreateDataset(unittest.TestCase):
@@ -7,8 +7,9 @@ class TestCreateDataset(unittest.TestCase):
         try:
             dataset = create_dataset("llama3chinese")
             print(dataset.title)
-            assert(dataset.title=="llama3chinese")
+            assert dataset.title == "llama3chinese"
             upload_file("README.md", "test/README.md", dataset)
+            delete_dataset(dataset)
         except Exception as err:
             print(f"An error occurred: {err}")
 
